;JAWS 8.x Script Manager Scripts
; Copyright 2010-2013, Freedom Scientific, Inc.

include "hjconst.jsh"
include "HJGlobal.jsh"
Include "jscript.jsm"
Include "common.jsm"

const
;Control ID's:
	cidInsertFunctionDescription = 6037 ,
	cidParameterDescription = 6060,
	cIDFunctionNameList = 6022,
	Back = 0,
	Forward = 1,
;Following are not translated, keystrokes within Script Manager and
;not language-specific.
	ksNextScript = "F2",
	ksPriorScript = "Shift+F2"
Globals
;for determining if the function name in the list has changed:
	string gsPrevFunctionName,
;for determining when the function description text changes:
	string gsPrevFunctionDescription,
;for scheduling speaking of the Insert Function description:
	int giScheduledSpeakInsertFunctionDescription
; BH global variables
globals
int iVerbosity, ; for storing the user's preference of speaking the number
string LineNumber ; for capturing the line number



Function AutoStartEvent ()
let iVerbosity = IniReadInteger ("Line Number Verbosity", "LineNumberVerbosity", 1, "Script Manager.ini")
; a value of 1 is stored here if the user hasn't selected anything, meaning that line numbers will be spoken as a default.



EndFunction

Script  ScriptFileName()
ScriptAndAppNames(msgScriptManagerAppName)
EndScript

void function ScheduledSpeakInsertFunctionDescription()
var
	string sDescription
let giScheduledSpeakInsertFunctionDescription = 0
let sDescription = GetWindowTextEx(FindDescendantWindow(GetParent(GetFocus()),cidInsertFunctionDescription),0,0)
;only spweak the description if it has changed:
if StringCompare(gsPrevFunctionDescription,sDescription) != 0 then
	Say(sDescription,ot_line)
EndIf
let gsPrevFunctionDescription =  sDescription
EndFunction

Void Function SayNonHighlightedText (handle hwnd, string Buffer)
var
	string sRealName
let sRealName = GetWindowName(GetRealWindow (GetFocus()))
if StringContains(sRealName,wn_InsertFunction) then
	if GetControlID(hwnd) == cidInsertFunctionDescription then
		;we want to announce the description only if it changes.
		;because the description text is received by SayNonHighlightedText one line at a time,
		;we wait until the description is finished writing then determine if the description ahs changed.
		;we unschedule and reschedule the delayed speaking function as long as the description text is being updated.
		if giScheduledSpeakInsertFunctionDescription then
			UnscheduleFunction(giScheduledSpeakInsertFunctionDescription)
		EndIf
		let giScheduledSpeakInsertFunctionDescription = ScheduleFunction("ScheduledSpeakInsertFunctionDescription",5)
		return
	endif
endif
if StringContains(sRealName,wn_ScriptInformation) 
|| StringContains(sRealName,wn_AssignToHotkey) then
	if GetControlID(hwnd) == cidParameterDescription then
		Say(Buffer, ot_buffer)
		return
	endif
endif
SayNonHighlightedText(hwnd, Buffer)
EndFunction

void Function SayHighLightedText (handle hwnd, string buffer)
if (hWnd == getFocus () && getWindowSubtypeCode (hwnd) == WT_EDITCOMBO) then
	; keep edit combos such as Category from double-speaking.
	Return
endIf
If GlobalMenuMode > 1
&& IsMSAAWindow(hWnd) then
	If hWnd != GetCurrentWindow() then
		Return;Let ActiveItemChangedEvent do its job.
	EndIf
EndIf
if GetScreenEcho() then
	if GetControlID(hWnd) == cIDFunctionNameList then
		;only announce the function name if it has changed:
		if StringCompare(buffer,gsPrevFunctionName) != 0 then
			Say(buffer, OT_HIGHLIGHTED_SCREEN_TEXT)
			let gsPrevFunctionName = buffer
		EndIf
		return
	EndIf
	;Say(buffer, OT_HIGHLIGHTED_SCREEN_TEXT) ; May cause double-speaking, call default:
endIf
return SayhighlightedText (hwnd, buffer)
EndFunction

int Function HandleCustomWindows (handle hWnd)
var
	int nSubType,
	string sRealName
If ! DialogActive () ||
InHjDialog () ||
MenusActive () then
	Return HandleCustomWindows (hWnd)
EndIf
Let nSubtype = GetWindowSubtypeCode (hWnd)
Let sRealName = GetWindowName (GetRealWindow (hWnd))
If (! sRealName) then
	If (nSubtype == WT_STATIC ||
	nSubtype == WT_DIALOG) then;Neither of these on focus should speak
		;SayWindowTypeAndText on the real window (HandleCustomRealWindows) already handles this appropriately.
		Return TRUE;Do nothing.
	EndIf
EndIf
Return HandleCustomWindows (hWnd)
EndFunction

void Function SayFocusedWindow ()
var
	handle hWnd,
	string sRealName
If SayFocusedHJDialogWindow(GlobalFocusWindow) then
	return
EndIf
let hWnd = GetFocus()
let sRealName = GetWindowName(GetRealWindow(hWnd))
let gsPrevFunctionDescription = cscNull
if sRealName == wn_ScriptInformation
|| sRealName == wn_NewScript then
	if GetControlID(hWnd) == cidInsertFunctionDescription then
		SayMessage (ot_control_name, msgAssignToHotKey_L,msgAssignToHotKey_S)
		return
	endif
endif
SayFocusedWindow ()
EndFunction

script NextScript ()
;TypeCurrentScriptKey()
TypeKey (ksNextScript)
Delay(3)
SayLine()
EndScript

script PriorScript ()
;TypeCurrentScriptKey()
TypeKey (ksPriorScript)
Delay(3)
SayLine()
EndScript

Void Function ProcessBoundaryStrike (handle hWnd, int edge)
	Return
EndFunction

void Function FindPrevOrNext (int iDirection)
If iDirection == Back then
	TypeKey (ksFindPrev)
Else
	TypeKey (ksFindNext)
EndIf
If DialogActive () ||
MenusActive () ||
UserBufferIsActive () ||
GetObjectTypeCode () != WT_EDIT then;
	Return Null();
EndIf
Delay(1)
If ! (GetCharacterAttributes ()&ATTRIB_HIGHLIGHT) then
	SayMessage (OT_ERROR, msgSearchError)
	Return;
EndIf
EndFunction

Script FindPrev ()
SayCurrentScriptKeyLabel ()
FindPrevOrNext (Back)
EndScript

Script FindNext ()
SayCurrentScriptKeyLabel ()
FindPrevOrNext (Forward)
EndScript





Script BasicSayLineNumber ()
; Assigned to control shift L.
InvisibleCursor ()
RouteInvisibleToPC ()
FindString (GetFocus(), "Line:", s_bottom, s_unrestricted, true)
NextWord ()
var
string SLineNumber
let SLineNumber = GetWord ()
PCCursor ()
Saystring ("Line "+SLineNumber)



 EndScript

; now for the automatic reading when the up and down arrow keys are pressed


Function GetLineNumber ()
InvisibleCursor ()
RouteInvisibleToPC ()
FindString (GetFocus(), "Line:", s_bottom, s_unrestricted, true)
NextWord ()
let LineNumber = getword ()
PCCursor ()
if GetVerbosity () == "Beginner" then 
SayUsingVoice (VCTX_Message, "line "+LineNumber, ot_user_requested_information)
else
SayUsingVoice (VCTX_Message, LineNumber, ot_user_requested_information)
EndIf



EndFunction

Script ToggleLineNumberSpeaking ()
; assigned to control shift T.
; also refer to AutostartEvent for details of reading the ini file for the user's verbosity preference.
; also refer to global variables
if IVerbosity == 0 then 
let iVerbosity = 1
SayUsingVoice (VCTX_Message, "Line numbers will be spoken", ot_user_requested_information)
else
let iVerbosity = 0
SayUsingVoice (VCTX_Message, "Line numbers will not be spoken", ot_user_requested_information)
EndIf
; now we have the preference, lets store it.
IniWriteInteger ("Line Number Verbosity", "LineNumberVerbosity", iVerbosity, "Script Manager.ini", true)



EndScript

Script SayNextLine ()
; no keystroke assigned as it is not necessary.
; now lets go to the next line which is the default.
PerformScript SayNextLine ()
pause () ; just to make sure we have actually gone to the next line.
if IsPCCursor ()
&& !MenusActive ()
&& IVerbosity == 1 then ; the user is not in a menu, the PC cursor is active and she has selected to use our special function.
GetLineNumber ()
EndIf



EndScript

Script SayPriorLine()
; no keystroke assigned as it is not necessary.
; now lets go to the previous line which is the default.
PerformScript SayPriorLine ()
pause () ; just to make sure we have actually gone to the previous line.
if IsPCCursor ()
&& !MenusActive ()
&& IVerbosity == 1 then ; the user is not in a menu, the PC cursor is active and she has selected to use our special function.
GetLineNumber ()
EndIf



EndScript

Script SayLine()
; no keystroke assigned as it is not necessary.
; however slightly different treatment is required not in the previous two scripts.
; lets read the current line
PerformScript SayLine ()
; we don't need a pause statement here as we are not moving.
 if IsPCCursor ()
&& !MenusActive ()
&& IVerbosity == 1 then ; the user is not in a menu, the PC cursor is active and she has selected to use our special function.
GetLineNumber ()
EndIf



EndScript

; now we have scripts for examination of control ID's.


Script Tab ()
PerformScript tab () ; default functionality
pause ()
var
handle h,
string WindowName
if GetCurrentControlID ()== 1136 then 
SayUsingVoice (VCTX_Message, "this is a combo box where you can select a file type", ot_user_requested_information)
elif GetCurrentControlID ()== 1148 then 
SayUsingVoice (VCTX_Message, "this is an edit field into which the file name should be typed", ot_user_requested_information)
; now lets vary it a little
elif GetCurrentControlID ()== 1 then  ; lets be careful because the ID of the open button is 1 and so is the OK button when composing a new script with control E.
let h = GetFocus()
let h = GetRealWindow(H)
let WindowName = GetWindowName(H)
if WindowName == "Open User File" then ; we're in the right window, the open dialog so lets go for it!
SayUsingVoice (VCTX_Message, "this is an open button, press space bar", ot_user_requested_information)
EndIf ; end of whether we are in the right window.
EndIf ; end of examination of Control ID values.


EndScript
Script ShiftTab ()
PerformScript Shifttab () ; default functionality
pause ()
var
handle h,
string WindowName
if GetCurrentControlID ()== 1136 then 
SayUsingVoice (VCTX_Message, "this is a combo box where you can select a file type", ot_user_requested_information)
elif GetCurrentControlID ()== 1148 then 
SayUsingVoice (VCTX_Message, "this is an edit field into which the file name should be typed", ot_user_requested_information)
; now lets vary it a little
elif GetCurrentControlID ()== 1 then   
; lets be careful because the ID of the open button is 1 and so is the OK button when composing a new script with control E.
; so we need to differentiate between them and make sure we only speak this information when in this dialog.
let h = GetFocus()
let h = GetRealWindow(H)
let WindowName = GetWindowName(H)
if WindowName == "Open User File" then ; we're in the right window, the open dialog so lets go for it!
SayUsingVoice (VCTX_Message, "this is an open button, press space bar", ot_user_requested_information)
EndIf ; end of whether we are in the right window.
EndIf ; end of examination of Control ID values.


EndScript

; the next few keystrokes will set focus to controls.
; control shift 1 will be the file name edit field.
; control shift 2 will be the files of type combo.


Script FileNameField ()
; assigned to control shift 1.
var
handle h
let h = GetFocus ()
let h = GetRealWindow(H)
let h = FindWindowWithClassAndId (h, "Edit", 1148)
SetFocus (h)


EndScript


Script FilesOfTypeField ()
; assigned to control shift 2.
var
handle h
let h = GetFocus ()
let h = GetRealWindow(H)
let h = FindWindowWithClassAndId (h, "", 1136)

SetFocus (h)






EndScript

