include "HJConst.jsh"
include "MSAAConst.jsh"
include "common.jsm"
include "ie.jsm"

const
	cwc_OperationStatusWindow = "OperationStatusWindow"

globals
;for the parent object name in DirectUIhWND class windows when we must keep track of it:
	string sDirectUIhWND_GroupName


string function GetFocusGroupName()
var
	int iLevel,
	int iTopLevel,
	string sName
SaveCursor()
PCCursor()
let iLevel = 1
let iTopLevel = GetAncestorCount()
while iLevel <= iTopLevel
	if GetObjectSubtypecode(true,iLevel) == wt_GroupBox then
		let sName = GetObjectName(true,1)
		;enable breaking out of the loop early by setting iLevel to iTopLevel:
		let iLevel = iTopLevel
	endIf
	let iLevel = iLevel+1
EndWhile
RestoreCursor()
return sName
EndFunction

void function ProcessEventOnFocusChangedEventEx(
	handle hwndFocus, int nObject, int nChild,
	handle hwndPrevFocus, int nPrevObject, int nPrevChild,
	int nChangeDepth, string sClass, int nType)
var
	string sGroupName,
	string sPrevGroupName
if sClass == cwc_DirectUIhWND then
	if GetWindowClass(GetTopLevelWindow(hwndFocus)) == cwc_OperationStatusWindow
	&& nChangeDepth > 1
	&& nType == wt_Button then
		UIARefresh(true)
	EndIf
	let sPrevGroupName = sDirectUIhWND_GroupName
	let sGroupName = GetFocusGroupName()
	let sDirectUIhWND_GroupName = sGroupName
else
	let sDirectUIhWND_GroupName = cscNull
EndIf
if sGroupName
&& sGroupName == sPrevgroupName then
	SayObjectActiveItem()
	return
EndIf
if nChangeDepth == -1 then
	if GetAppMainWindow(hWndFocus) != GetAppMainWindow(hwndPrevFocus) then
		FocusChangedEvent(hwndFocus,hwndPrevFocus)
		return
	endIf
endIf
ProcessEventOnFocusChangedEventEx(hwndFocus, nObject, nChild, hwndPrevFocus, nPrevObject, nPrevChild, nChangeDepth, sClass, nType)
EndFunction

void Function FocusChangedEventProcessAncestors(handle FocusWindow, handle PrevWindow)
var
	int nLevel,
	string sName
if GetWindowClass(FocusWindow) == cwc_DirectUIhWND
&& GetWindowClass(GetTopLevelWindow(FocusWindow)) == cwc_OperationStatusWindow then
	let nLevel = GetFocusChangeDepth()
	while nLevel >= 0
		if nLevel > 1
		&& !GetObjectSubtypecode(true,nLevel) then
			let sName = GetObjectName(true,nLevel)
			if sName
			&& StringRight(GetObjectName(true,nLevel-1),StringLength(sName)) == sName then
				;skip over announcing this redundant information:
				let nLevel= nLevel-1
			EndIf
		EndIf
		sayObjectTypeAndText(nLevel)
		let nLevel= nLevel-1
	EndWhile
	return
EndIf
FocusChangedEventProcessAncestors(FocusWindow,PrevWindow)
EndFunction

Int Function HandleCustomAppWindows(handle hWnd)
var
 	string sWndName,
	handle hFocus,
 	handle hAppWnd
if GetWindowClass(hWnd) == cWc_dlg32770
&& IsWinVista() then
 	let sWndName = GetWindowName(hWnd)
 	if StringCompare(sWndName,wn_DeleteFile) == 0
 	|| StringCompare(sWndName,wn_DeleteFolder) == 0
 	|| StringCompare(sWndName,wn_DeleteShortcut) == 0 then
 		SayWindowTypeAndTextForNonstandardDialogs (hWnd, sWndName)
 		return true
 	EndIf
EndIf
;shlwapi.dll as desktop does not have a window title,
;but we say that it is the desktop anyway:
if GetAppFileName () == sApp_shlwapi then
	let hFocus = GetFocus()
	let hAppWnd = GetAppMainWindow(hFocus)
	if !GetWindowName(hAppWnd)
	&& GetWindowClass(hAppWnd) == cwc_WorkerW
	&& GetWindowClass(GetParent(hFocus)) == cwc_ShellDll_DefView
	&& !GetWindowName(GetRealWindow(hFocus))
	&& GetWindowSubtypeCode(hFocus) == wt_ListView then
		Say(cmsgAppName_shlwapi,OT_APP_START)
		return true
	endIf
endIf
Return HandleCustomAppWindows(hWnd)
EndFunction

void Function SayWindowTypeAndTextForNonstandardDialogs (handle hwnd, string name)
var
	string sMessage
let sMessage = FormatString (msgDialog1_L, name)
Say(sMessage,ot_dialog_name)
let hwnd = GetFirstChild (hwnd)
while hwnd
	if GetWindowSubTypeCode (hwnd) == wt_static
	|| GetWindowClass(hWnd) == cwc_SysLink then
		SayWindow (hwnd, read_everything)
	EndIf
	let hwnd = GetNextWindow (hwnd)
EndWhile
EndFunction

void function SayWindowTitleForApplication(handle hAppWnd, handle hRealWnd,
	handle hCurWnd, int iTypeCode)
;shlwapi.dll as desktop does not have a window title,
;but we say that it is the desktop anyway:
if GetAppFileName () == sApp_shlwapi then
	if !GetWindowName(hAppWnd)
	&& GetWindowClass(hAppWnd) == cwc_WorkerW
	&& GetWindowClass(GetParent(GetFocus())) == cwc_ShellDll_DefView
	&& !GetWindowName(hRealWnd)
	&& iTypeCode == wt_ListView then
		SayMessage(ot_USER_REQUESTED_INFORMATION,
			FormatString (cmsg29_L, cmsgAppName_shlwapi, cscNull),
			FormatString (cMsg29_S, cmsgAppName_shlwapi, cscNull))
		return
	endIf
endIf
SayWindowTitleForApplication(hAppWnd, hRealWnd, hCurWnd, iTypeCode)
EndFunction

void function SayObjectTypeAndText(optional int nLevel)
var
	object oTree,
	object oItem,
	handle hWnd,
	int nType
If InHomeRowMode()
|| !IsPcCursor()
|| UserBufferIsActive() then
	SayObjectTypeAndText (nLevel)
	Return
EndIf
let hWnd = GetFocus()
if GetWindowClass(hWnd) == cwc_DirectUIhWND
&& GetWindowClass(GetTopLevelWindow(hWNd)) == cwc_OperationStatusWindow then
	;By checking the type first,
	;we ensure that the accessible tree is only built when these types are to be announced:
	let nType = GetObjectSubtypecode(true,nLevel)
	if nType == wt_button
	|| nType == wt_Link then
		let oTree = GetUIAObjectTree(hWnd)
		if oTree then
			;speak the static text that goes with these controls:
			if nType == wt_Button then
				let oItem = oTree.FindByAutomationId("eltConflictInterruptDescription")
			elif nType == wt_Link then
				let oItem = oTree.FindByKeyboardFocus(1).PriorSibling
				if oItem.Role != ROLE_SYSTEM_STATICTEXT then
					let oItem = Null()
				EndIf
			EndIf
			if oItem then
				Say(oItem.Name,ot_dialog_text)
				SayObjectActiveItem()
				return
			EndIf
		EndIf
	EndIf
EndIf
SayObjectTypeAndText(nLevel)
EndFunction

Script GetFileProperties()
; assigned to control shift P.
; lets set everything up
var
string text,
string output, ; to display information in the virtual viewer if the keystroke is pressed twice quickly.
handle h
let output = "Properties." ; virtual viewer title
let h = GetFocus ()
let h = GetRealWindow (h)
let h = FindWindow (h, "", "Size on disk:") ; name of window next to the one we want.
let h = GetNextWindow (h)
let text = GetWindowText (h, false) ; this is the value of the file.
var
int isegment
let iSegment = StringSegmentIndex (text, text, "(", false) 
let text = SubString (text, 1, iSegment-1)
saystring ("file size")
let output = output+"\r\n"+"File Size "+text ; check this carefully, \r\n means a new line.
saystring (text)
if IsSameScript () then 
StopSpeech ()
SayFormattedMessage (ot_user_buffer, output)
SayFormattedMessage (ot_user_buffer, "Press Escape to return to the Properties Dialog box.")
endif
braillestring (output)


EndScript


